@@ -1,7 +1,7 @@
-use holo_hash::{EntryHashB64, HeaderHashB64};
 use hdk::prelude::*;
+use holo_hash::{EntryHashB64, HeaderHashB64};
 
-use crate::retrieval::EntryAndHash;
+use crate::retrieval::retrieval::EntryAndHash;
 
 #[doc = "This data structure will be very broadly useful and represents
           how an entry should be serialized along with what metadata to
@@ -18,11 +18,11 @@ pub struct WireElement<T> {
 }
 
 impl<T> From<EntryAndHash<T>> for WireElement<T> {
-  fn from(e: EntryAndHash<T>) -> Self {
-    Self {
-      entry: e.0,
-      header_hash: HeaderHashB64::new(e.1),
-      entry_hash: EntryHashB64::new(e.2)
+    fn from(e: EntryAndHash<T>) -> Self {
+        Self {
+            entry: e.0,
+            header_hash: HeaderHashB64::new(e.1),
+            entry_hash: EntryHashB64::new(e.2),
+        }
     }
-  }
-}
\ No newline at end of file
+}
